/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

// RecoverSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "uiDlg.h"
#include "RecoverSettings.h"
#include "pim/utils.h"
#include "pim/Maincpp.h"
#include "base/startcmd.h"
#include "pim/SettingFunctions.h"
#include "pim/ClientSettings.h"
#include "pim/account.h"
#include "ClientUtil.h"
#include "notify/timed_msgbox.h"
#include "HwndFunctions.h"
#include "winreg.h"
#include <vector>
#include "localizationUtils.h"

static int checkRefreshFromServer(HWND wndUI, vector<const wchar_t*> sources);
static wstring getCommandLine(vector<const wchar_t*> sources);

// CRecoverSettings dialog
void writeSourceForSlow(const wchar_t*  value);
IMPLEMENT_DYNAMIC(CRecoverSettings, CDialog)

CRecoverSettings::CRecoverSettings(CWnd* pParent /*=NULL*/)
    : CDialog(CRecoverSettings::IDD, pParent)
{

}

CRecoverSettings::~CRecoverSettings()
{
}

void CRecoverSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CRecoverSettings, CDialog)
    ON_WM_SIZE()
	ON_WM_CTLCOLOR()
END_MESSAGE_MAP()

// CRecoverSettings message handlers

BOOL CRecoverSettings::OnInitDialog()
{
    CString s1, s2;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER); SetWindowText(s1);
    CDialog::OnInitDialog();

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_RECOVER;
    #if defined WIN32_PLATFORM_WFSP
    #if (_WIN32_WCE < 0x500)
        dlgCommandBar.nToolBarId = IDR_MENUBAR_RECOVER;
    #endif
    #endif
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    // set ppc header
#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER); SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif

	s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RESTORE_CONTINUE);
	SetDlgItemText(IDC_RECOVER_MESSAGE, s1);
    GetDlgItem(IDC_RECOVER_MESSAGE)->SetFont(&fontBold);

    EnableRecoverButton(true);

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    return FALSE;
}

void CRecoverSettings::OnOK()
{
    wstring command;
    int nCheckedSources = 0;
    int firstSourceToSyncId = -1;
    vector<const wchar_t*> sourceToSync;

	command += L"refresh-from-server ";

    if(firstSourceToSyncId == -1) {             //not set
		firstSourceToSyncId = SOURCE_TASKS;
    }
    sourceToSync.push_back(TASKS_NAME);
    nCheckedSources++;
    ((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl().setState(SOURCE_TASKS, ITEM_STATE_TO_SYNC);
    
    if(nCheckedSources != 0){
        wstring sourceList = getCommandLine(sourceToSync);

        command += sourceList;

        // hide recover window
        ShowWindow(SW_HIDE);

        // start recover sync
        int check  = checkStartSync();
        if(check == 0) {
            BOOL toSync = TRUE;
            if (command.find(L"refresh-from-server") != wstring::npos) {
                if (checkRefreshFromServer(GetSafeHwnd(), sourceToSync) != 0)
                    toSync = FALSE;
            }

            if (toSync) {
                // set first source to sync
                ((CuiDlg*)AfxGetMainWnd())->setFirstSourceToSync(firstSourceToSyncId);
                // start sync
                ((CuiDlg*)AfxGetMainWnd())->refreshSourcePane(SOURCE_TASKS);
                ((CuiDlg*)AfxGetMainWnd())->StartSync(command.c_str());
            }
        }
    }
  EndDialog(RECOVER_SYNC_STARTED);
}

void CRecoverSettings::EnableRecoverButton(bool bEnable){
    HWND hwndMB = SHFindMenuBar (this->GetSafeHwnd());
    TBBUTTONINFO tbi;
    if (!hwndMB)
        return;

    memset (&tbi, 0, sizeof (tbi));
    tbi.cbSize = sizeof (tbi);
    tbi.dwMask = TBIF_STATE;
    ::SendMessage (hwndMB, TB_GETBUTTONINFO, IDOK, (LPARAM)&tbi);

    if(bEnable)
        tbi.fsState = TBSTATE_ENABLED;
    else
        tbi.fsState &= ~TBSTATE_ENABLED;

    ::SendMessage (hwndMB, TB_SETBUTTONINFO, IDOK, (LPARAM)&tbi);
}

bool CRecoverSettings::isCheckboxChecked()
{
	return true;
}

#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void CRecoverSettings::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
    int resId;
    CString s1,s2;

    switch(DRA::GetDisplayMode())
    {
    case DRA::Landscape:
        resId = IDD_UI_RECOVER_WIDE; break;
    case DRA::Portrait:
        resId = IDD_UI_RECOVER; break;
    case DRA::Square:
        resId = IDD_UI_RECOVER_SQUARE; break;
    default:
        resId = IDD_UI_RECOVER; break;
    };

    DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));
    // set ppc header
#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER); SetDlgItemText(IDC_HEADER_STATIC, s1);
#endif

	s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RESTORE_CONTINUE); SetDlgItemText(IDC_RECOVER_MESSAGE, s1);
}
#endif

wstring getCommandLine(vector<const wchar_t*> sources) {

    wstring sourceList;
    vector<const wchar_t*>::iterator sourcesIterator;
    int i = 1;
    for (sourcesIterator = sources.begin();
            sourcesIterator != sources.end();
            sourcesIterator++) {

        sourceList.append(*sourcesIterator);
        if (sources.size() - i != 0) {
            sourceList.append(TEXT(","));
        }
        i++;
    }

    return sourceList;
}

int checkRefreshFromServer(HWND wndUI, vector<const wchar_t*> sources) {

    wstring sourceList;
    int i = 1, ret = 0;

    vector<const wchar_t*>::iterator sourcesIterator;

    for (sourcesIterator = sources.begin();
            sourcesIterator != sources.end();
            sourcesIterator++) {

        sourceList.append(getFriendlyName(*sourcesIterator));

        if (sources.size() - i == 1) {
            sourceList.append(TEXT(" and "));
        } else if (sources.size() - i != 0) {
            sourceList.append(TEXT(", "));
        }
        i++;
    }

    HwndFunctions::closePreviousMsgBox();
    int val = TimedMessageBox(wndUI, getLocalizationUtils()->getLocalizationString(IDS_RS_ARMSG1), getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                         MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, ASK_REFRESH_TIMEOUT * 1000); // default ASK_TIMEOUT = 10

    if (val == IDYES) {
        LOG.debug("User said yes to delete local %S and refresh.", sourceList.c_str());
    }
    else {
        LOG.debug("User said no to delete local %S and refresh." , sourceList.c_str());
        ret = -1;
    }

    return ret;
}

HBRUSH CRecoverSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}