/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#include "stdafx.h"
#include "ui.h"
#include "uiDlg.h"
#include "CommunicationSettings.h"

#include "LoglevelSettings.h"
#include "RecoverSettings.h"
#include "About.h"
#include "TaskSettings.h"
#include "pim/ClientSettings.h"
#include "pim/maincpp.h"
#include "shellapi.h"

#include "SynchronizationMessages.h"

#include "Shellapi.h"
#include "aygshell.h"
#include "pim/SettingFunctions.h"
#include "ClientUtil.h"

#include "base/util/utils.h"
#include <windows.h>

#include "comutil.h"
#include "HwndFunctions.h"
#include "notify/timed_msgbox.h"

#include "CustomButton.h"
#include "ClientUtil.h"

#include "TaskSettings.h"
#include "DeviceInfo.h"
#include "pim/account.h"
#include "processUtils.h"

#define ITEMS_FROM_SERVER 20

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

long retValueLastSync=0;
bool isServerConfigured();
void trim(wstring& str);
wstring reformatDate(wstring date);
wstring reformatTime(wstring time);

CuiDlg::CuiDlg(CWnd* pParent /*=NULL*/)
: CDialog(CuiDlg::IDD, pParent)
{
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CuiDlg::OnMenuComm()
{
    isInSettings = true;
    CCommunicationSettings wndCommSett;
    INT_PTR result = wndCommSett.DoModal();
    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
}

void CuiDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_LIST1, lstSources);
}


BEGIN_MESSAGE_MAP(CuiDlg, CDialog)
#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
    ON_WM_SIZE()
#endif
    ON_COMMAND(IDM_COMM, OnMenuComm)
    //}}AFX_MSG_MAP
    ON_COMMAND(IDM_LOGLEVEL, &CuiDlg::OnMenuLoglevel)
    ON_COMMAND(ID_SYNC, &CuiDlg::OnSync)
    ON_COMMAND(ID_MENU_EXIT, &CuiDlg::OnMenuExit)
    ON_COMMAND(ID_RECOVER, &CuiDlg::OnMenuRecover)
    ON_MESSAGE(ID_MYMSG_CONNECTING, &CuiDlg::OnMsgConnecting)
    ON_MESSAGE(ID_MYMSG_ENDING, &CuiDlg::OnMsgEndingSync)
    ON_MESSAGE(ID_MYMSG_ENDING_SOURCE, &CuiDlg::OnMsgEndingSource)
    ON_MESSAGE(ID_MYMSG_STARTSYNC_ENDED, &CuiDlg::OnStartsyncEnded)
    ON_MESSAGE(ID_MYMSG_STARTING_SYNC, &CuiDlg::OnStartingSync)
    ON_MESSAGE(ID_MYMSG_STARTING_SOURCE, &CuiDlg::OnStartingSource)
    ON_MESSAGE(ID_MYMSG_ITEM_SYNCED_FROM_SERVER, &CuiDlg::OnMsgItemSyncedFromServer)
    ON_MESSAGE(ID_MYMSG_ITEM_SYNCED_FROM_CLIENT, &CuiDlg::OnMsgItemSyncedFromClient)
    ON_MESSAGE(ID_MYMSG_TOTAL_ITEMS, &CuiDlg::OnMsgTotalItems)
    ON_COMMAND(ID_MENU_ABOUT, &CuiDlg::OnMenuAbout)
    ON_COMMAND(IDABORT, &CuiDlg::OnMenuStop)
    ON_COMMAND(ID_SETTINGS_SYNC, &CuiDlg::OnMenuSync)
    ON_WM_CLOSE()
    //ON_WM_CONTEXTMENU() // for context menu
    ON_MESSAGE(ID_MYMSG_SOURCE_STATE, &CuiDlg::OnMsgSourceState)
    ON_NOTIFY(LVN_ITEMCHANGED, IDC_LIST1, &CuiDlg::OnItemchangedList1)
    #if defined(WIN32_PLATFORM_PSPC)
//        ON_COMMAND(ID_MENU_CONTEXT_SYNC, &CuiDlg::OnContextMenuSync)
//        ON_COMMAND(ID_MENU_CONTEXT_SETTINGS, &CuiDlg::OnContextMenuSettings)
    #endif
    ON_COMMAND(ID_TOOLS_DEVICEINFO, &CuiDlg::OnToolsDeviceinfo)
END_MESSAGE_MAP()


// CuiDlg message handlers

BOOL CuiDlg::OnInitDialog()
{

    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(),IDS_FUNAMBOL); SetWindowText(s1);
    CDialog::OnInitDialog();
    LOG.setLevel(getLOGLevel());
    AfxGetMainWnd()->EnableWindow();
    // init settings
    ClientSettings* cs = getRegConfig();

    try {
        // Set the icon for this dialog.  The framework does this automatically
        //  when the application's main window is not a dialog
        SetIcon(m_hIcon, TRUE);			// Set big icon
        SetIcon(m_hIcon, FALSE);		// Set small icon
    }
    catch(...) {
        LOG.error("uiDlg: can't set application icon.");
    }
    // set items size, check dpi
    /*double dpiY = (double)GetDeviceCaps(GetDC()->GetSafeHdc(),LOGPIXELSY)/96.0;
    int height = lstSources.getItemHeight();
    height = (int) height * dpiY;
    lstSources.setItemHeight(height);*/

     // try to set the font used in main window
    try {
        LOGFONT lf;
        memset(&lf, 0, sizeof(LOGFONT));
        HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
        GetObject(hFont, sizeof lf, &lf);
        lf.lfWeight = FW_BOLD;
        lf.lfHeight += FONT_HEIGHT_OFFSET;
        VERIFY(lstSources.fontBold.CreateFontIndirect(&lf));
    }
    catch(...) {
        LOG.error("uiDlg: can't set main screen font.");
    }

    // check if another sync is in progress
    if(checkStartSync() != 0)
    {
        CreateMenuBar(IDM_STOP);
        #if defined WIN32_PLATFORM_WFSP
        #if (_WIN32_WCE < 0x500)
            CreateMenuBar(IDR_MENUBAR_STOP);
        #endif
        #endif
        bSyncStarted = true;
    }
    else
    {
        CreateMenuBar(IDR_MENU_MAIN);
        #if defined WIN32_PLATFORM_WFSP
        #if (_WIN32_WCE < 0x500)
            CreateMenuBar(IDR_MAINFRAME);
        #endif
        #endif
        bSyncStarted = false;
    }

    initSyncControls();
    lstSources.SetItemState(0, LVIS_SELECTED, LVIF_STATE);

    // stretch the list to cover all client area.
    // this is used only for smartphone because it has only one dialog.
    // For pocket pc it is overwritten by the OnSize function
    CRect rect, rectImage;
    GetClientRect(&rect);
    lstSources.setItemWidth(rect.Width());

	int pad = 200, img_w = 188, img_h = 83;
	
	#if defined(WIN32_PLATFORM_WFSP)
		#if defined(_DEVICE_RESOLUTION_AWARE)
		pad = DRA::SCALEY(130);
		#else
		pad = 130;
		#endif
		img_w = 130;
		img_h = 57;
    #endif

    lstSources.SetWindowPos(GetActiveWindow(),0, pad, rect.Width(), rect.Height() + pad,
        SWP_SHOWWINDOW );
    lstSources.Invalidate();

	GetDlgItem(IDC_IMG_FUNLOGO)->GetClientRect(&rectImage);
	int x = rect.Width() / 2 - rectImage.Width() / 2;
	GetDlgItem(IDC_IMG_FUNLOGO)->SetWindowPos(GetActiveWindow(), x, 10, img_w, img_h, SWP_SHOWWINDOW);
	GetDlgItem(IDC_IMG_FUNLOGO)->Invalidate();

    currentSource = 0;
    resetCurrentItem = true;
    lastItem = 0;
    currentItem = 0;
    totalItems = 0;
    sourceEnds = 0;
    sourceBegins = 0;
    nMailsToSync = 0;

    isInSettings = false;
    firstSourceId = -1;

    // set states
    syncSourceContactState = SYNCSOURCE_STATE_OK;
    syncSourceCalendarState = SYNCSOURCE_STATE_OK;
    syncSourceTaskState = SYNCSOURCE_STATE_OK;
    syncSourceNoteState = SYNCSOURCE_STATE_OK;
    syncSourceBriefcaseState = SYNCSOURCE_STATE_OK;
    syncSourceMailState = SYNCSOURCE_STATE_OK;
    bSyncCanceled = false;

    CString user;
    CString password;
    user = getRegConfig()->getAccessConfig().getUsername();
    password = getRegConfig()->getAccessConfig().getPassword();

    if( (!(isServerConfigured())) ||user.IsEmpty() || password.IsEmpty()){
       // OnMenuComm();
        isInSettings = true;
        CCommunicationSettings commSettings;
        CString msg;
        
        msg.LoadString(getLocalizationUtils()->getLocaleResource(),IDS_ERROR_ENTER_CREDENTIALS);
        commSettings.showMessage(msg);
        INT_PTR result = commSettings.DoModal();
        isInSettings = false;
        return TRUE;
    }
    
    return TRUE;
}
//////////////////////////////////////////////////////////////////////////


#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void CuiDlg::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
    int resId, x, y;

    switch(DRA::GetDisplayMode())
    {
    case DRA::Landscape:
		y = 200;
        resId = IDD_UI_DIALOG_WIDE;
        break;
    case DRA::Portrait:
		y = 200;
        resId = IDD_UI_DIALOG; break;
    case DRA::Square:
		y = 120;
        resId = IDD_UI_DIALOG_SQUARE; break;
    default:
		y = 200;
        resId = IDD_UI_DIALOG; break;
    };

    DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));

	y = DRA::SCALEY(y);

    // stretch the list to cover all client area
    CRect rect, rectImage;
    GetClientRect(&rect);
    lstSources.setItemWidth(rect.Width());
    lstSources.SetWindowPos(GetActiveWindow(), 0, y, rect.Width(), rect.Height() + y,
        SWP_SHOWWINDOW );
    lstSources.Invalidate();

	GetDlgItem(IDC_IMG_FUNLOGO)->GetClientRect(&rectImage);
	x = (rect.Width() / 2) - (rectImage.Width() / DRA::SCALEX(2));
	GetDlgItem(IDC_IMG_FUNLOGO)->SetWindowPos(GetActiveWindow(), x, DRA::SCALEY(10), 188, 83, SWP_SHOWWINDOW);
	GetDlgItem(IDC_IMG_FUNLOGO)->Invalidate();

}
#endif

void CuiDlg::OnMenuLoglevel()
{
    isInSettings = true;
    CLoglevelSettings wndLoglevel;
    INT_PTR result = wndLoglevel.DoModal();
    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
}

void CuiDlg::OnSync()
{
    if (!lstSources.getIsLocked()){
        lockPanes();
    }

    // start a sync of all enabled sources
    lstSources.setStateToAll(ITEM_STATE_TO_SYNC, true);
    lstSources.SetItemState(lstSources.GetCurSelItem(), 0, LVIS_FOCUSED);
    refreshSourcePane(SOURCE_TASKS);
    setFirstSourceToSync();
    StartSync(TEXT("manual"));
}

void CuiDlg::OnClose()
{
    int processID = 0;
    HWND wnd = NULL;
    wnd = ::FindWindow(TEXT("funambolstartsync"), NULL);
    if ((processID = checkStartSync()) != 0)  {
        // sync is in progress
        HwndFunctions::closePreviousMsgBox();
        
        // send msg to startsync to stop
        if(wnd){
            ::PostMessage(wnd, WM_COMMAND, (WPARAM)10, NULL);
        }
        Sleep(500);
        DWORD code = 0;
        GetExitCodeProcess((HANDLE)processID, &code);
        if (code == STILL_ACTIVE) {
            HANDLE hProcess = OpenProcess(0, FALSE, processID);
            if (hProcess) {
                startProgram(SYNCAPP, TEXT("removeNotif"));
                TerminateProcess(hProcess, -6); //-6 code for the user stop sync.
                CloseHandle(hProcess);
            }
        }
    }
    else {        
        startProgram(SYNCAPP, TEXT("removeNotif"));        
    }
    EndDialog(0);
}

void CuiDlg::OnMenuExit()
{
    int ret = EndAppQuery(GetSafeHwnd());
    if((ret == 0) || (ret== 1))
        //CDialog::OnOK();
        getRegConfig()->saveDirty();
        EndDialog(0);
    }

void CuiDlg::OnMenuRecover()
{
    CRecoverSettings wndRecover;
    // TODO: workaround for isInSettings
    //isInSettings = true;
    INT_PTR result = wndRecover.DoModal();
    if(result == RECOVER_SYNC_STARTED){
        // the user started a recover sync
    }
    isInSettings = false;
}


LRESULT CuiDlg::OnStartsyncEnded(WPARAM wParam, LPARAM lParam)
{
    CString s1("");

    lstSources.stopAnim(currentSource);

    getRegConfig()->read();

    if(!isInSettings){
        CreateMenuBar(IDR_MENU_MAIN);
        #if defined WIN32_PLATFORM_WFSP
            #if (_WIN32_WCE < 0x500)
                CreateMenuBar(IDR_MAINFRAME);
            #endif
        #endif
    }

    retValueLastSync = lParam;
    bSyncStarted = false;

    // stop sync icon animation
    for(int i=0;i<lstSources.GetItemCount();i++){
        lstSources.stopAnim(lstSources.indexToId(i));
    }

    if( (retValueLastSync == 0) || (retValueLastSync == 10003) ||
        (retValueLastSync == -20) ||  (retValueLastSync == -6) ){
        // for these errorcodes we don't display error message
    }
    else{
        CString message;  message.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CODE_ERROR_NOT_DONE);
        CString errormsg = SyncErrorMsg(retValueLastSync);

        HwndFunctions::closePreviousMsgBox();
        HWND wndFocus = GetFocus()->GetSafeHwnd();
        if(errormsg != ""){
            TimedMessageBox(GetSafeHwnd(), errormsg, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
             MB_OK | MB_ICONHAND | MB_SETFOREGROUND, INFINITE);
        }
        else{
            TimedMessageBox(GetSafeHwnd(), message, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
            MB_OK | MB_ICONHAND | MB_SETFOREGROUND, INFINITE);
        }
        ::SetFocus(wndFocus);
    }

    // reset variables
    currentSource = 0;
    resetCurrentItem = true;
    totalItems = 0;
    lastItem = 0;
    sourceEnds = 0;
    sourceBegins = 0;
    firstSourceId = -1;
    nMailsToSync = 0;

    lstSources.SetItemState(lstSources.GetCurSelItem(), 0, LVIS_FOCUSED);

    //refreshRecoveredSources();
    unlockPanes();

    // for the case of scheduled sync when the user is in a config window
    if(!isInSettings){
        refreshSourcePanes();
    }

    if(! bSyncCanceled) // TODO: is this necessary?
        lstSources.setStateToAll(ITEM_STATE_OK);

    return 0;
}

LRESULT CuiDlg::OnStartingSync(WPARAM wParam, LPARAM lParam)
{
    CString s2;

    if(!isInSettings){
        HwndFunctions::closePreviousMsgBox();

        CreateMenuBar(IDM_STOP);
        #if defined WIN32_PLATFORM_WFSP
        #if (_WIN32_WCE < 0x500)
            CreateMenuBar(IDR_MENUBAR_STOP);
        #endif
        #endif
    }
    lockPanes();

    bSyncStarted = true;

    //getRegConfig()->saveDirty();
    // received start sync message, we check if startsync.exe has given the first source to be synced
    // if wParam !=0, then in lParam we have the id of the 1st source
    if(wParam != 0){
        firstSourceId = (int)lParam;
    }

    return 0;
}

void CuiDlg::OnMenuAbout()
{
    CAbout wndAbout;
    isInSettings = true;
    INT_PTR result = wndAbout.DoModal();
    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
}

void CuiDlg::OnMenuStop()
{
    // because the sync is possibly being canceled in EndAppQuery, we set bSyncCanceled to true before calling it
    bSyncCanceled = true;
    int ret = EndAppQuery(GetSafeHwnd());
    // If startsync was not running simulate the message.
    // 1 - sync not running
    // 0 - user choosed to stop the sync, we need to call OnStartsyncEnded
    //       to reset the controls
    if(ret== 0) {
        bSyncCanceled = true;
        refreshCanceledState();
        OnStartsyncEnded(NULL, (LPARAM) -20 ); // nothing to show...
    }
    else if (ret== 1){
        bSyncCanceled = true;
        refreshCanceledState();
        OnStartsyncEnded(NULL, (LPARAM) -6 );
    }
    else{
        bSyncCanceled = false;
    }
}

void CuiDlg::OnMenuSync()
{
	CTaskSettings wndSett;

    isInSettings = true;
	INT_PTR result = wndSett.DoModal();
    isInSettings = false;

    refreshSourcePanes();
}

LRESULT CuiDlg::OnStartingSource(WPARAM wParam, LPARAM lParam)
{
    CString s1,s2;

    if (!lstSources.getIsLocked()){
        lockPanes();
    }

    lstSources.stopAnim(currentSource);
    currentSource = lParam;

    if(! (sourceBegins & (int)pow(2.0, currentSource)) ){

        // set the bit for the current source to know that for it we have received a Sync Source Ended event
        sourceBegins |= (int) pow(2.0, currentSource);

        CString msg;
        msg.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CHECKING);
        msg+=" ";

        switch(lParam)
        {
        case SOURCE_MAIL:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_MAIL); break;
        case SOURCE_CALENDAR:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CALENDAR); break;
            //s1=getStringLang(IDS_LOWER_CALENDAR); break;
        case SOURCE_CONTACTS:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CONTACTS); break;
        case SOURCE_TASKS:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_TASKS); break;
        case SOURCE_BRIEFCASE:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_FILES); break;
        case SOURCE_NOTES:
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_NOTES); break;

        default:
            s1 = "";
        }

        msg+=s1;

        lstSources.setName(lParam, msg);
    };

    // if source fails without sending a message that it has finished
    // then we send it ourselves
    if(lstSources.getIsSyncing(currentSource)){
        OnMsgEndingSource(NULL, NULL);
    }

    // if the source fails without sending 'Starting' message
    //s1 = lstSources.getText(currentSource);
    s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CONNECTING);
    s1 = lstSources.getName(firstSourceId);
    if(s1.Compare(s2) == 0){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FAILED);
        lstSources.setName(firstSourceId, s1);
    }

    //currentSource = lParam;
    currentItem = 0;
    lstSources.stopAnim(firstSourceId);

    lstSources.setIsSyncing(currentSource, true);
    lstSources.SetCurSelItem(lstSources.idToIndex(currentSource));
    lstSources.startAnim(currentSource);

    return 0;
}

// not a WM_APP message handler
void CuiDlg::OnMsgItemSynced(WPARAM wParam, LPARAM lParam, bool fromClient)
{
    int source=-1;
    CString msg("");
    CString s1;

    if(!bSyncStarted)
        return;

    currentItem++;

    if( (wParam > 0)  && (currentSource != wParam) )  // extra check
    {
        resetCurrentItem = true;
        lastItem = 0;
    };



    if( fromClient ){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SENDING);
        msg+=s1; msg+=" ";
    }
    else{
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECEIVING);
        msg+=s1; msg+=" ";
    }

    source = wParam;
    currentSource = source;

    if(currentSource == SOURCE_MAIL){
        totalItems = nMailsToSync;
    };

    if( (!resetCurrentItem) && (lParam != 0) )  // the server sent more valid noc's for the same source
        currentItem =lastItem+lParam;


    switch(source)
    {
    case SOURCE_MAIL:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_MAIL); break;
    case SOURCE_CALENDAR:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CALENDAR); break;
    case SOURCE_CONTACTS:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CONTACT); break;
    case SOURCE_TASKS:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_TASKS); break;
    case SOURCE_BRIEFCASE:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_FILE); break;
    case SOURCE_NOTES:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_NOTE); break;

    default:
        s1 = "";
    }

    msg+=s1;
    if(msg == "")
        goto finally;
    msg+=" ";

    char* v = ltow(currentItem);
    msg+=v;
    delete [] v; v = NULL;

    if( (totalItems != -1) && (currentItem <= totalItems))
    {
        msg+="/";
        char* v = ltow(totalItems);
        msg+=v;
        delete [] v; v = NULL;
    }

    lstSources.setName(source, msg);

   finally:

    return;
}

LRESULT CuiDlg::OnMsgItemSyncedFromServer( WPARAM wParam, LPARAM lParam)
{
    OnMsgItemSynced(wParam,lParam, false);
    return 0;
}

LRESULT CuiDlg::OnMsgItemSyncedFromClient( WPARAM wParam, LPARAM lParam)
{
    OnMsgItemSynced(wParam, lParam, true);
    return 0;
}


// TODO: remove unused code
LRESULT CuiDlg::OnMsgTotalItems(WPARAM wParam, LPARAM lParam)
{
    CString msg;
    CString s1;
    bool changeLabel = false;
    int sourceNumber = wParam;
    totalItems = lParam;

    msg.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECEIVING); msg+=" ";

    switch(sourceNumber)
    {
    case SOURCE_MAIL:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_MAIL); break;
    case SOURCE_CALENDAR:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CALENDAR); break;
    case SOURCE_CONTACTS:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_CONTACTS); break;
    case SOURCE_TASKS:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_TASKS); break;
    case SOURCE_BRIEFCASE:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_NOTES); break;
    case SOURCE_NOTES:
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LOWER_FILES); break;

    default:
        s1 = "";
    }

    msg+=s1;
    if( (msg!="") &&  changeLabel) // !
    {
        SetDlgItemText(IDC_MAIN_MSG,msg);
        lstSources.setName(wParam, msg);
        if(lstSources.GetNextItem(-1, LVNI_SELECTED ) != wParam)
            lstSources.SetItemState(wParam, LVIS_SELECTED, LVIF_STATE);
    }


    if(wParam == SOURCE_MAIL){
        nMailsToSync+=totalItems;
    }

    return 0;
}


void CuiDlg::CreateMenuBar(const int id){
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = id;
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!\n\n"));
    }
}

void CuiDlg::OnOK()
{
    this->ShowWindow(SW_MINIMIZE);
}

bool isServerConfigured() {

    ClientSettings* cs = getRegConfig();
    CString s1;
    s1 = cs->getAccessConfig().getSyncURL();
    if(s1.Find(_T("<")) > 0)
        return false;
    else
        return true;

}

// override back key sph
void CuiDlg::OnCancel(){
    // just hide the window
    this->ShowWindow(SW_MINIMIZE);
}


HBRUSH CuiDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor){
    if(pWnd->GetRuntimeClass() == RUNTIME_CLASS(CCustomButton)){
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetBkColor(RGB(255,255,255));
        return (HBRUSH) GetStockObject(HOLLOW_BRUSH);
    }

    HBRUSH hbr = (HBRUSH) CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
    return hbr;
}


void CuiDlg::initSyncControls(){
    CString s1;
    ClientSettings* cs = getRegConfig();
    s1 = cs->getAccessConfig().getSyncURL();

    // add sync sources to list
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TASKS);
    lstSources.addItem(SOURCE_TASKS, s1, AfxGetApp()->LoadIcon(IDI_TASKS));

    refreshSourcePanes();

    lstSources.Invalidate();
}

void CuiDlg::OnContextMenu(CWnd* /*pWnd*/, CPoint point)
{
    // TODO:  handle 2nd event and close the popup
    CPoint mPointCurrent;
    ::GetCursorPos(&mPointCurrent);
    CMenu mPopupMenu;
//    mPopupMenu.LoadMenu(IDR_MENU_CONTEXT);


    CMenu *pContextMenu = mPopupMenu.GetSubMenu(0);
    /*pContextMenu->TrackPopupMenu(TPM_LEFTALIGN |
    TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD |
    TPM_NONOTIFY,
    mPointCurrent.x, mPointCurrent.y, GetActiveWindow(),
    NULL); */

    pContextMenu->TrackPopupMenu(TPM_LEFTALIGN |
        TPM_RETURNCMD | TPM_NONOTIFY,
        point.x, point.y, GetActiveWindow(),
        NULL);
}

wstring CuiDlg::getFormattedDateAndTime(long lastSync) {

    SYSTEMTIME timeDest; wchar_t data[80]; 
    wchar_t formatDate[80]; wchar_t formatTime[80];

    CTime timeSyncContacts(lastSync);
    
    timeSyncContacts.GetAsSystemTime(timeDest);
    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLONGDATE, data, 80);

    wstring result = reformatDate(data);   
    GetDateFormat(LOCALE_USER_DEFAULT, NULL, &timeDest, result.c_str(), formatDate, 80); 
    
    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_STIMEFORMAT, data, 80);
    result = reformatTime(data);   

    GetTimeFormat(LOCALE_USER_DEFAULT, NULL, &timeDest, result.c_str(), formatTime, 80); 
    
    result = formatDate;
    result += TEXT(" ");
    result += formatTime;    
    
    return result;
}


void CuiDlg::refreshSourcePane( int id ){
    CString s1;
    ClientSettings* cs = getRegConfig();
    bool enable;
    unsigned long lastSync=0;
    wstring date_time;

    if(id == SOURCE_TASKS){
        lastSync=0;

        enable =  strcmp(cs->getConfigSourcesParameter("task","sync"), "none") != 0;
        lstSources.enableItem(id, enable);
        lastSync = cs->getConfigSourcesLastTimestamp("task");

		s1.LoadStringW(getLocalizationUtils()->getLocaleResource(), IDS_READY_TO_SYNC);
		lstSources.setName(SOURCE_TASKS, s1);

        if(syncSourceTaskState == SYNCSOURCE_STATE_NOT_SYNCED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_FAILED);
            lstSources.setText(SOURCE_TASKS, s1);
        }
        else if( syncSourceTaskState == SYNCSOURCE_STATE_CANCELED){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_CANCELED);
            lstSources.setText(SOURCE_TASKS, s1);
        }
        else if(lastSync == 0) {
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOT_SYNCHRONIZED);
            lstSources.setText(SOURCE_TASKS, s1);
        }
        else
        {           
            date_time = getFormattedDateAndTime(lastSync);
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCHRONIZED); s1+= TEXT(" ");
            s1 += date_time.c_str();
            lstSources.setText(SOURCE_TASKS, s1);
        }
    }

    EnableSyncButton(isSyncEnabled());
    lstSources.resetSourceIcon(id);
}

void CuiDlg::refreshSourcePanes(){
    refreshSourcePane(SOURCE_TASKS);

    ClientSettings* cs = getRegConfig();

    lstSources.resetSourceIcons();
}


void CuiDlg::SelectSource( int index ){
 if(index < 0)
    return;

 try{
    CString s1,s2;
    bool refreshPane = false;
    ClientSettings* cs = getRegConfig();

    int id = lstSources.indexToId(index);

    if(id == SOURCE_MAIL){
        if(!doesFunambolAccountExist()){
            refreshSourcePane(SOURCE_MAIL);
        }
       else{

        // check if source is enabled
        if( ( strcmp(cs->getConfigSourcesParameter("mail","sync"), "none") != 0) ){
            // start sync with this source
            lstSources.setState(SOURCE_MAIL, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_MAIL);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual mail"));
        }
        else{
            if(!doesFunambolAccountExist())
                return;
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_EMAIL); s1+=s2; s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("mail","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_MAIL);
                getRegConfig()->saveDirty();
            }
        }
      }
    };

    if(id == SOURCE_CONTACTS){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("contact","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_CONTACTS, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_CONTACTS);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual contact"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CONTACTS); s1+=s2; s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
              MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("contact","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_CONTACTS);
                getRegConfig()->saveDirty();
            }
        }
    };

    if(id == SOURCE_CALENDAR){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("calendar","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_CALENDAR, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_CALENDAR);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual calendar"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CALENDAR); s1+=s2; s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("calendar","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_CALENDAR);
                getRegConfig()->saveDirty();
            }
        }
    };

    if(id == SOURCE_TASKS){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("task","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_TASKS, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_TASKS);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual task"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TASKS); s1+=s2; s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("task","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_TASKS);
                getRegConfig()->saveDirty();
            }
        }
    };

    if(id == SOURCE_BRIEFCASE){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("briefcase","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_BRIEFCASE, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_BRIEFCASE);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual briefcase"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_BRIEFCASE); s1+=s2; s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("briefcase","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_BRIEFCASE);
                getRegConfig()->saveDirty();
            }
        }
    };

#if defined(WIN32_PLATFORM_PSPC)
    if(id == SOURCE_NOTES){
        // check if source is enabled
        if( strcmp(cs->getConfigSourcesParameter("note","sync"), "none") != 0 ){
            // start sync with this source
            lstSources.setState(SOURCE_NOTES, ITEM_STATE_TO_SYNC);
            setFirstSourceToSync(SOURCE_NOTES);
            refreshSourcePane(SOURCE_MAIL);
            StartSync(_T("manual note"));
        }
        else{
            // ask if to enable source
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ENABLE_SYNC_FOR); s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOTES); s1+=s2; s1+=" ";
            s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_ITEMS_LOWER); s2+="?";
            s1+=s2;

            int retMsgBox =TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

            if(retMsgBox == IDYES){
                refreshPane = true;
                cs->setConfigSourcesParameter("note","sync","two-way");
                getRegConfig()->setDirty(CS_DIRTY_SOURCE_NOTES);
                getRegConfig()->saveDirty();
            }
        }
    };
#endif


    if(refreshPane){
        refreshSourcePane(id);
    }

  }
  catch(...){
      LOG.error("uiDlg: error while selecting source");
  }

  lstSources.UpdateWindow();
}

void CuiDlg::StartSync(CString param){
    CString s1;

    int check  = checkStartSync();
    int ret;

    bSyncCanceled = false;

    // check credentials
    ClientSettings* cs = getRegConfig();
    CString user;
    CString password;
    user = cs->getAccessConfig().getUsername();
    password = cs->getAccessConfig().getPassword();

    if( user.IsEmpty() || password.IsEmpty()){
        HwndFunctions::closePreviousMsgBox();
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_ENTER_CREDENTIALS);
        TimedMessageBox(GetSafeHwnd(), s1, _T("MilkSync alert"), MB_OK | MB_ICONHAND, INFINITE);
        OnStartsyncEnded(0,0); // signal sync ended
        OnMenuComm();
        return; // we return from this so we don't lock the panes
    }
    else
      if (!isServerConfigured()) {
        HwndFunctions::closePreviousMsgBox();
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_SET_FUNAMBOL_URL);
        TimedMessageBox(GetSafeHwnd(), s1, _T("MilkSync alert"), MB_OK | MB_ICONHAND, INFINITE);
        OnStartsyncEnded(0,0); // signal sync ended
        OnMenuComm();
        return; // we return from this so we don't lock the panes

    } else if(check == 0) {
        // maybe the user deleted the mail account
        // so we refresh the mail and if there is no source to be synced, we return
        if(! isSyncEnabled())
            return; // no sync will be performed

        ret = startProgram(SYNCAPP, param);	 // start sync process
        if ( (ret == 0) || (ret == -5) )
        {
            HwndFunctions::closePreviousMsgBox();
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TEXT_STARTSYNC_NOT_FOUND);
            TimedMessageBox(GetSafeHwnd(), s1,_T("MilkSync alert"), MB_OK | MB_ICONHAND, INFINITE);
            OnStartsyncEnded(0,0); // signal sync ended
            return; // we return from this so we don't lock the panes
        }
        else
        {
            dlgCommandBar.nToolBarId = IDM_STOP;
            #if defined WIN32_PLATFORM_WFSP
            #if (_WIN32_WCE < 0x500)
                CreateMenuBar(IDR_MENUBAR_STOP);
            #endif
            #endif

            if(!SHCreateMenuBar(&dlgCommandBar)){
                TRACE(_T("Cannot create command bar!"));
                return ;
            }
        }
    };

    lstSources.SetItemState(-1, LVIS_SELECTED, LVIF_STATE);
    // lock panes so they cannot be selected
    lockPanes();
}

void CuiDlg::lockPanes(){
    lstSources.lockList();
}

void CuiDlg::unlockPanes(){
    lstSources.unlockList();
}

BOOL CuiDlg::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if( ( (pMsg->message == WM_KEYDOWN) || (pMsg->message == WM_KEYUP) ||
        ((pMsg->message == WM_LBUTTONDOWN) && (pMsg->hwnd == lstSources.GetSafeHwnd())) )  // click in source list
        && (lstSources.getIsLocked()) ){
            bProcessed = true;
            return TRUE;
    };

    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){
            bProcessed = true;
            SelectSource(lstSources.GetNextItem(-1, LVNI_SELECTED ));
        }
    };

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

void CuiDlg::CancelSync(){
    OnMenuStop();
}

void CuiDlg::refreshCanceledState(){
    if(lstSources.getState(SOURCE_MAIL) == ITEM_STATE_TO_SYNC)
        syncSourceMailState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_CONTACTS) == ITEM_STATE_TO_SYNC)
        syncSourceContactState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_CALENDAR) == ITEM_STATE_TO_SYNC)
        syncSourceCalendarState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_TASKS) == ITEM_STATE_TO_SYNC)
        syncSourceTaskState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_BRIEFCASE) == ITEM_STATE_TO_SYNC)
        syncSourceBriefcaseState = SYNCSOURCE_STATE_CANCELED;

    if(lstSources.getState(SOURCE_NOTES) == ITEM_STATE_TO_SYNC)
        syncSourceNoteState = SYNCSOURCE_STATE_CANCELED;
}
/*
void CuiDlg::refreshRecoveredSources(){
    ClientSettings* cs = getRegConfig();

    if(checkStartSync()){

    }

    if(strstr(cs->getConfigSourcesParameter("mail","sync"), "refresh"))
        cs->setConfigSourcesParameter("mail","sync","two-way");
    if(strstr(cs->getConfigSourcesParameter("contact","sync"), "refresh"))
        cs->setConfigSourcesParameter("contact","sync","two-way");
    if(strstr(cs->getConfigSourcesParameter("calendar","sync"), "refresh"))
        cs->setConfigSourcesParameter("calendar","sync","two-way");
    if(strstr(cs->getConfigSourcesParameter("task","sync"), "refresh"))
        cs->setConfigSourcesParameter("task","sync","two-way");
    if(strstr(cs->getConfigSourcesParameter("briefcase","sync"), "refresh"))
        cs->setConfigSourcesParameter("briefcase","sync","two-way");
    if(strstr(cs->getConfigSourcesParameter("note","sync"), "refresh"))
        cs->setConfigSourcesParameter("note","sync","two-way");

}
 */

// source id as wParam, state as lParam
LRESULT CuiDlg::OnMsgSourceState( WPARAM wParam, LPARAM lParam){
    if(wParam == SOURCE_MAIL)
        syncSourceMailState = lParam;
    if(wParam == SOURCE_CONTACTS)
        syncSourceContactState = lParam;
    if(wParam == SOURCE_CALENDAR)
        syncSourceCalendarState = lParam;
    if(wParam == SOURCE_TASKS)
        syncSourceTaskState = lParam;
    if(wParam == SOURCE_BRIEFCASE)
        syncSourceBriefcaseState = lParam;
    if(wParam == SOURCE_NOTES)
        syncSourceNoteState = lParam;

    return 0;
}

void CuiDlg::setFirstSourceToSync(){
    CArray<int> sources;
    bool found = false;
    /* RTM
	sources.Add(SOURCE_MAIL); sources.Add(SOURCE_CONTACTS);
    sources.Add(SOURCE_CALENDAR);
	*/
	sources.Add(SOURCE_TASKS);
	/* RTM
    #if defined(WIN32_PLATFORM_PSPC)
        sources.Add(SOURCE_NOTES);
    #endif
    sources.Add(SOURCE_BRIEFCASE);
	*/
    for(int i=0; (i<sources.GetCount()) && (!found); i++){
        if(lstSources.isEnabledItem(sources[i])){
            firstSourceId = sources[i];
            found = true;
        }
    }

}

LRESULT CuiDlg::OnMsgEndingSource( WPARAM , LPARAM ){
    // sync source ended event
    CString s1;
    lastItem = currentItem;
    currentItem = 0;

    if(sourceEnds & (int)pow(2.0, currentSource) ){
        // done receiving
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_DONE_RECEIVING);
        lstSources.setName(currentSource, s1);
    }
    else{
        // done sending
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_DONE_SENDING);
        lstSources.setName(currentSource, s1);
        // set the bit for the current source to know that for it we have received a Sync Source Ended event
        sourceEnds |= (int) pow(2.0, currentSource);
        // reset nr of mails
        nMailsToSync =0;
    }

    lstSources.setIsSyncing(currentSource, false);
    return 0;
}

//
LRESULT CuiDlg::OnMsgConnecting( WPARAM , LPARAM ){
    CString s1;

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CONNECTING);
    if(firstSourceId == -1){
        setFirstSourceToSync();
    }

    // check if we have a valid first source
    if(firstSourceId != -1){
        lstSources.SetCurSelItem(lstSources.idToIndex(firstSourceId));
        lstSources.setName(firstSourceId, s1);
		lstSources.setText(firstSourceId, TEXT(""));
        lstSources.startAnim(firstSourceId);
    }

    return 0;
}

// 'Ending...' sync message
LRESULT CuiDlg::OnMsgEndingSync( WPARAM , LPARAM ){
    CString s1;

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ENDING);
	lstSources.setName(currentSource, s1);
    lstSources.startAnim(currentSource);

    return 0;
}

void CuiDlg::OnItemchangedList1(NMHDR* pNMHDR, LRESULT* pResult)
{
	*pResult = 0;
	return;
/* RTM
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

    int id = lstSources.indexToId(pNMListView->iItem);
    // if item is disabled we do not change icon

    if(lstSources.isEnabledItem(id)){
        // Only interested in state changes
        if (pNMListView->uChanged & LVIF_STATE)
            if (pNMListView->uNewState & LVIS_FOCUSED)
            {
                // new selection
                lstSources.setIcon(id, AfxGetApp()->LoadIcon(IDI_ARROWS22A));
            }
            else if (pNMListView->uOldState & LVIS_FOCUSED)
            {
                // deselection
                lstSources.resetSourceIcon(id);
            }
    }

    lstSources.InvalidatePane(id);

   *pResult = 0;
	*/
}

// context menu handlers
void CuiDlg::OnContextMenuSync(){
    // TODO: implementation
}

void CuiDlg::OnContextMenuSettings(){
    // TODO: implementation
}

void CuiDlg::OnToolsDeviceinfo()
{
    CDeviceInfo wndDeviceInfo;
    isInSettings = true;
    INT_PTR result = wndDeviceInfo.DoModal();
    isInSettings = false;
    if(bSyncStarted){
        CreateMenuBar(IDM_STOP);
    }
}


void CuiDlg::EnableSyncButton(bool bEnable){
    HWND hwndMB = SHFindMenuBar (this->GetSafeHwnd());
    TBBUTTONINFO tbi;
    if (!hwndMB)
        return;

    memset (&tbi, 0, sizeof (tbi));
    tbi.cbSize = sizeof (tbi);
    tbi.dwMask = TBIF_STATE;
    ::SendMessage (hwndMB, TB_GETBUTTONINFO, ID_SYNC, (LPARAM)&tbi);

    if(bEnable){
        tbi.fsState = TBSTATE_ENABLED;
    }
    else{
        tbi.fsState &= ~TBSTATE_ENABLED;
    }

    ::SendMessage (hwndMB, TB_SETBUTTONINFO, ID_SYNC, (LPARAM)&tbi);
}

bool CuiDlg::isSyncEnabled(){
	return true;
	/* RTM
    bool enabled = false;
    enabled = lstSources.isEnabledItem(SOURCE_MAIL) ||
              lstSources.isEnabledItem(SOURCE_CONTACTS) ||
              lstSources.isEnabledItem(SOURCE_CALENDAR) ||
              lstSources.isEnabledItem(SOURCE_TASKS) ||
              lstSources.isEnabledItem(SOURCE_BRIEFCASE);

    #if defined(WIN32_PLATFORM_PSPC)
        enabled |= lstSources.isEnabledItem(SOURCE_NOTES);
    #endif

    return enabled;
	*/
}


void trim(wstring& str)
{
  wstring::size_type pos = str.find_last_not_of(' ');
  if(pos != string::npos) {
    str.erase(pos + 1);
    pos = str.find_first_not_of(' ');
    if(pos != string::npos) str.erase(0, pos);
  }
  else str.erase(str.begin(), str.end());
}

wstring reformatDate(wstring date) {
    LANGID lId = GetUserDefaultLangID(); 
    lId &= 0x1F;

	if (lId == LANG_JAPANESE) {
		return date;
	}

    int found = 0;
    // remove "dddd" or "dddd," or "dddd, "
    if ((found = date.find(TEXT("dddd, "))) != wstring::npos) {
        date.replace(found, 6, TEXT(""));
    } else if ((found = date.find(TEXT("dddd,"))) != wstring::npos) {
        date.replace(found, 5, TEXT(""));
    }else if ((found = date.find(TEXT("dddd"))) != wstring::npos) {
        date.replace(found, 4, TEXT(""));
    }
    
    trim(date);
    // change full month in abbreviation (e.g. September into Sept)    
    if ((found = date.find(TEXT("MMMM, "))) != wstring::npos) {
        date.replace(found, 6, TEXT("MMM, "));
    } else if ((found = date.find(TEXT("MMMM,"))) != wstring::npos) {
        date.replace(found, 5, TEXT("MMM,"));
    } if ((found = date.find(TEXT("MMMM"))) != wstring::npos) {
        date.replace(found, 4, TEXT("MMM"));
    }
    
    trim(date);
    // remove year
    if (((found = date.find(TEXT("yyyy, "))) != wstring::npos) || 
        ((found = date.find(TEXT(", yyyy"))) != wstring::npos)) {
        date.replace(found, 6, TEXT(""));
    } else if (((found = date.find(TEXT("yyyy,"))) != wstring::npos) ||
               ((found = date.find(TEXT(",yyyy"))) != wstring::npos)) {
        date.replace(found, 5, TEXT(""));
    } if ((found = date.find(TEXT("yyyy"))) != wstring::npos) {
        date.replace(found, 4, TEXT(""));
    }
    trim(date);
    
    return date;

}


wstring reformatTime(wstring time) {
    
    int found = 0;
    // remove ":ss" or ":ss," or "ss"
    if ((found = time.find(TEXT(":ss,"))) != wstring::npos ||
        (found = time.find(TEXT(".ss,"))) != wstring::npos) {
        time.replace(found, 4, TEXT(""));
    } else if ((found = time.find(TEXT(":ss"))) != wstring::npos ||
               (found = time.find(TEXT(".ss"))) != wstring::npos) {
        time.replace(found, 3, TEXT(""));
    }else if ((found = time.find(TEXT("ss"))) != wstring::npos) {              
        time.replace(found, 2, TEXT(""));
    }
    
    trim(time);   
    
    return time;

}
