/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

/* notlstnr.cpp : Defines the entry point for the DLL application.
*                  Handle all the interaction with Services.exe and
*                  with the netwotk. Uses class S4JProxy to interact
*                  with sync4j engine.
*/

#include "stdafx.h"
#include "notlstnr.h"
#include "Notify.h"
#include <stdio.h>
#include <service.h>
#include <iptypes.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include <Pmpolicy.h>

#include "http/GPRSConnection.h"
#include "s4jproxy.h"
#include "worker.h"
#include "notify/checknet.h"
#include "notify/s4n_service.h"
#include "notify/addresschange.h"
#include "notify/util.h"
#include "pim/SettingFunctions.h"
#include "base/startcmd.h"
#include "base/util/utils.h"
#include "processUtils.h"

#include "CTPConfig.h"
#include "CTPManager.h"
#include "CTPMessage.h"


#define NOTLSTNR_LOG_NAME       "notlstnr.txt"
#define NOTLSTNR_LOG_PATH       ""
#define NOTLSTNR_LOG_TITLE      "Funambol Windows Mobile Notification-Listener Log"


// Service state
static DWORD g_dwServiceState = SERVICE_STATE_UNINITIALIZED;
// Global critical section
static CRITICAL_SECTION g_cs;

//static
DWORD addressChangeHandler(void);
static DWORD acceptConnection(SOCKET sock);

int getPushTypeFromRegistry();


HANDLE moduleHandle = 0;

///////////////////////////////////////////////////////////////////
// Notification Listener: Dll entry
///////////////////////////////////////////////////////////////////
/**
* Dll entry point.
*/
extern "C" BOOL WINAPI DllMain(HANDLE hInstDll, DWORD fdwReason,
        LPVOID lpvReserved) {

    LogLevel l;

    switch (fdwReason) {
        case DLL_PROCESS_ATTACH:
            g_dwServiceState = SERVICE_STATE_UNINITIALIZED;
            InitializeCriticalSection (&g_cs);
            // This DLL does not require thread attatch/deatch
            DisableThreadLibraryCalls((HMODULE)hInstDll);
            
            // Init LOG:
            Log(1, NOTLSTNR_LOG_PATH, NOTLSTNR_LOG_NAME);
            LOG.reset(NOTLSTNR_LOG_TITLE);
            l = getLOGLevel();
            LOG.setLevel(l);

            LOG.info("Process attach.");
            moduleHandle = hInstDll;
            break;

        case DLL_PROCESS_DETACH:
            DeleteCriticalSection (&g_cs);
            LOG.info("Process detach.");
            moduleHandle = 0;
            break;
    }
    return TRUE;
}


///////////////////////////////////////////////////////////////////
// Notification Listener: service callbaks
///////////////////////////////////////////////////////////////////
/**
* Service callback: not implemented.
*/
APIEXPORT BOOL S4N_Open(
        DWORD dwData,
        DWORD dwAccess,
        DWORD dwShareMode
        ) {
    LOG.debug("Open - dwData: %lx, dwAccess: %lx, dwShareMode: %lx",
              dwData, dwAccess, dwShareMode);

    return TRUE;
}

/**
* Service callback: not implemented.
*/
APIEXPORT BOOL S4N_Close( DWORD dwData) {
    LOG.debug("Close - dwData: %lx", dwData);
    return TRUE;
}

/**
* Service callback: close the socket and exit.
*/
APIEXPORT BOOL S4N_Deinit( DWORD dwData) {
    LOG.info("Deinit - dwData: %lx", dwData);

    //MessageBox (NULL, TEXT("DeInit"), TEXT("Test"), MB_SETFOREGROUND | MB_OK);
    
    // Stop child threads of STP.
    stopWorker();

    // Stop CTP process: try to send the BYE message and 
    // then close the socket connection.
    CTPManager* ctpManager = CTPManager::getInstance();
    ctpManager->stopCTP();

    //stopCheckForNews();

    // Remove unattended mode
    PowerPolicyNotify(PPN_UNATTENDEDMODE, FALSE);

    return TRUE;
}

/**
* Service callback: init the service and return the listen port.
*/
APIEXPORT DWORD S4N_Init( DWORD dwData) {
    EnterCriticalSection (&g_cs);

    if (g_dwServiceState != SERVICE_STATE_UNINITIALIZED) {
        // Someone is trying to load multiple times
        LOG.error("S4ND: ERROR: service already initialized on S4N_Init() call");
        LeaveCriticalSection (&g_cs);
        return 0;
    }

    g_dwServiceState = SERVICE_STATE_STARTING_UP;
    LOG.info("S4ND: S4N_Init success - service is in starting up state");

    // Try to connect to Internet.
    EstablishConnection();
    Sleep(2000);

    LeaveCriticalSection (&g_cs);
    return getListenerPort();
}

/**
* Service callback: main method containing the service.exe events.
* On IOCTL_SERVICE_REFRESH and IOCTL_SERVICE_NOTIFY_ADDR_CHANGE events, it is invoked
* a new ip address change notification to the server.
* On IOCTL_SERVICE_CONNECTION event it is opened a socket connection listening
* for the server messages
*/
APIEXPORT BOOL S4N_IOControl(
        DWORD dwData,
        DWORD dwCode,
        PBYTE pBufIn,
        DWORD dwLenIn,
        PBYTE pBufOut,
        DWORD dwLenOut,
        PDWORD pdwActualOut
        )
{
    DWORD dwError = ERROR_INVALID_PARAMETER;

    //LOG.debug("IOControl - dwCode: %lx", dwCode);
    EnterCriticalSection (&g_cs);

    switch(dwCode){
        // Control code sent to start a service.  (services start FIN0:)
        case IOCTL_SERVICE_START:
            if (g_dwServiceState != SERVICE_STATE_OFF) {
                LOG.error("S4ND: ERROR: IOCTL_SERVICE_START fails because service is not off. State=<%d>", g_dwServiceState);
                dwError = ERROR_SERVICE_ALREADY_RUNNING;
            }
            else {
                // Notification listener must work in unattended mode
                PowerPolicyNotify(PPN_UNATTENDEDMODE, TRUE);

                LOG.debug("S4ND: State changed to ON");
                g_dwServiceState = SERVICE_STATE_ON;
                dwError = ERROR_SUCCESS;
            }
            break;

        // Control code sent to refresh a service. (services refresh FIN0:)
        case IOCTL_SERVICE_REFRESH:

            if (g_dwServiceState != SERVICE_STATE_ON) {
                LOG.error("S4ND:ERROR:IOCTL_SERVICE_REFRESH fails because service is not on. State=<%d>", g_dwServiceState);
                dwError = ERROR_SERVICE_NOT_ACTIVE;
            }
            else {
                LOG.debug("S4ND: Stop on a refresh");
                g_dwServiceState = SERVICE_STATE_SHUTTING_DOWN;
                stopWorker();
                // Wait for tasks to terminate
                Sleep(1000);
                // Shut the service down, re-read configuration
                // TODO: close connections? Abort sync?
                // (if we have any) and then restart.
                LOG.debug("S4ND: Restarting on refresh");
                g_dwServiceState = SERVICE_STATE_ON;
                dwError = ERROR_SUCCESS;
                addressChangeHandler();
            }
            break;

        // Control code sent to stop a service.  (services stop S4N0:)
        case IOCTL_SERVICE_STOP:
            // No longer accept new network connections.
            // Close existing connections
            if (g_dwServiceState != SERVICE_STATE_ON) {
                LOG.error("S4ND:ERROR:IOCTL_SERVICE_STOP fails.  Service state is not on. Current State=<%d>", g_dwServiceState);
                dwError = ERROR_SERVICE_NOT_ACTIVE;
            }
            else {
                LOG.debug("S4ND: Service stopping");
                stopWorker();
                g_dwServiceState = SERVICE_STATE_OFF;
                dwError = ERROR_SUCCESS;
            }
            break;

        // An application (possibly services.exe itself) is
        // querying for the service's running state.
        case IOCTL_SERVICE_STATUS:
            // No need for critical section since this is an atomic read.
            __try {
                if (pBufOut && dwLenOut == sizeof(DWORD)) {
                    *(DWORD *)pBufOut = g_dwServiceState;
                    if (pdwActualOut)
                        *pdwActualOut = sizeof(DWORD);
                    dwError = ERROR_SUCCESS;
                }
            }
            __except (EXCEPTION_EXECUTE_HANDLER) {
                LOG.error("S4ND: ERROR Invalid pointer on IOCTL_SERVICE_STATUS");
                dwError = ERROR_INVALID_PARAMETER;
            }
            break;
        // Services.exe has completed super-service init for this service.
        // Incoming super-service connections can come in at anytime now.
        case IOCTL_SERVICE_STARTED:
            if (g_dwServiceState != SERVICE_STATE_STARTING_UP) {
                LOG.error("S4ND:ERROR:IOCTL_SERVICE_STARTED failed because state != starting up. State=<%d>", g_dwServiceState);
                dwError = ERROR_SERVICE_ALREADY_RUNNING;
            }
            else {
                LOG.debug("S4ND: IOCTL_SERVICE_STARTED changed state to SERVICE_STATE_ON");
                g_dwServiceState = SERVICE_STATE_ON;
                dwError = ERROR_SUCCESS;
            }
            addressChangeHandler();
            break;

        // IP address changed: notify the server
        case IOCTL_SERVICE_NOTIFY_ADDR_CHANGE:
            dwError = addressChangeHandler();
            break;

        // A new connection request
        case IOCTL_SERVICE_CONNECTION:
            if (dwLenIn != sizeof(SOCKET)) {
                dwError = ERROR_INVALID_PARAMETER;
            }
            else {
                SOCKET sock;

                __try {
                    sock = * ((SOCKET*)pBufIn);
                }
                __except (EXCEPTION_EXECUTE_HANDLER) {
                    dwError = ERROR_INVALID_PARAMETER;
                    break;
                }
                // Call new connection handler
                dwError = acceptConnection(sock);
            }
            break;
        default:
            //LOG.debug("IOControl - dwCode: %lx", dwCode);
            return TRUE;
    }

    if (dwError != ERROR_SUCCESS)
        SetLastError(dwError);

    LeaveCriticalSection(&g_cs);

    return (dwError==ERROR_SUCCESS);
}

/**
* Service callback: not implemented.
*/
APIEXPORT DWORD S4N_Read(
        DWORD dwData,
        LPVOID pBuf,
        DWORD dwLen
        ) {
    LOG.debug("Read - dwData: %lx", dwData);
    return 0;
}

/**
* Service callback: not implemented.
*/
APIEXPORT DWORD S4N_Write(
        DWORD dwData,
        LPVOID pInBuf,
        DWORD dwInLen
        ) {
    LOG.debug("Write - dwData: %lx", dwData);
    return 0;
}


/**
* Service callback: not implemented.
*/
APIEXPORT DWORD S4N_Seek(
        DWORD dwData,
        long pos,
        DWORD type
        ) {
    LOG.debug("Seek - dwData: %lx", dwData);
    return 0;
}



///////////////////////////////////////////////////////////////////
// Notification Listener: connection handler
///////////////////////////////////////////////////////////////////
/**
* Open the socket connection and waits for the server messages.
*/
static DWORD acceptConnection(SOCKET sock)
{
    if (g_dwServiceState != SERVICE_STATE_ON) {
        // Do not accept connections unless the service is on

        // ALWAYS close the socket, even if you return
        // FALSE. Services.exe leaves it to the
        // service to close the socket in all cases.
        LOG.error("S4ND: ERROR: Service Conn fails. Service State != ON.  State = <%d>", g_dwServiceState);
        send(sock, RESP_BUSY, strlen(RESP_BUSY),0);
        closesocket(sock);
        return ERROR_SERVICE_NOT_ACTIVE;
    }
    int ret = ERROR_EXCEPTION_IN_SERVICE;

    __try {
        LOG.debug("Starting worker...");
        ret = startWorker(sock);
    }
    __except (EXCEPTION_EXECUTE_HANDLER) {
        LOG.info("Exception in Worker thread.");
    }

    if( ret != ERROR_SUCCESS ) {
        if (ret == ERROR_BUSY ){
            LOG.debug("Worker busy...");
            send(sock, RESP_BUSY, strlen(RESP_BUSY),0);
        }
        else {
            LOG.error("Worker error...");
            send(sock, RESP_GENERIC, strlen(RESP_BUSY),0);
        }
        closesocket(sock);
    }
    // Worker thread will close the socket.
    return ret;
}


///////////////////////////////////////////////////////////////////
// Address change handler
///////////////////////////////////////////////////////////////////

// FIXME: use the DM
/*
* Test a configuration property to understand if the client has already
* notified the server.
*/
static bool isServerNotified()
{
    wchar_t buf[10];

    memset(buf, 0, 10*sizeof(wchar_t));
    getClientConfigurationInternal (TEXT(""), PROPERTY_SPDM_SERVER_NOTIFIED,
                                     buf, FALSE);
    return ( wcscmp(buf, TEXT("1")) == 0);
}

// FIXME: use the DM
/*
* Set a configuration property to mark the client has
* notified the server.
*/
void setServerNotified(bool val)
{
    setClientConfiguration(
            TEXT(""),
            PROPERTY_SPDM_SERVER_NOTIFIED,
            ( (val) ? TEXT("1") : TEXT("0") ),
            NULL);
}

/**
 * Starts the STP or CTP process, based on the registry property 'push'.
 * STP: Start a particular sync saying to the server the new client IP address.
 * It sets the server notified parameter if succesfully finished.
 * CTP: starts the CTP main thread (ctpWorker) which will connect only
 * if the stpWorker ended unsuccessfully.
 */
DWORD addressChangeHandler(void)
{
    static wchar_t oldIP[40] = TEXT("127.0.0.1");
    wchar_t newIP[40];
    DWORD ret = ERROR_SUCCESS;
    bool useSTP = false;
    bool useCTP = false;

    //MessageBox (NULL, TEXT("addressChangeHandler"), TEXT("Test"), MB_SETFOREGROUND | MB_OK);

    // Read getPush -> switch if using STP / CTP
    int pushType = getPushTypeFromRegistry();
    if (pushType & 1) {
        useSTP = true;
    }
    if (pushType & 2) {
        useCTP = true;
    }

    if (!useSTP && !useCTP) {
        // Nothing to do
        setServerNotified(false);
        return 0;
    }
    LOG.debug("useSTP = %d, useCTP = %d", useSTP, useCTP);

    if (checkNetwork()) {
        LOG.debug("Connected to network, getting IP address...");
        getIPAddress(newIP);
        if (wcscmp(newIP, TEXT("192.168.55.101"))==0 ||
             wcsstr(newIP, TEXT("169.254.2.")) ) {
            LOG.info("Device is using ActiveSync -> use only CTP");
            wcscpy(oldIP, newIP);
            setServerNotified(false);
            useSTP = false;
            useCTP = true;
        }

        // The server is already notified and the IP is
        // the same of the current, do not send a new notification.
        if ( isServerNotified() && (wcscmp(oldIP, newIP)==0) ) {
            LOG.info("Server already notified.");
            return ret;
        }

        // Save the IP
        wcscpy(oldIP, newIP);
        LOG.info("New IP: %ls", newIP);

        HANDLE stpThread = NULL;
        CTPManager* ctpManager = CTPManager::getInstance();

        // ----------- STP -----------
        if (useSTP) {
            // create STP thread, get handle of stpThread
            stpThread = CreateThread(NULL, 0, stpWorker, NULL, 0, NULL);
            if (!stpThread) {
                LOG.error("Error creating STP thread: code 0x%08x", GetLastError());
            }
        }

        // ----------- CTP -----------
        if (useCTP) {
            // create CTP thread, passing handle stpThread (NULL if not created)
            ctpManager->startCTP(stpThread);
        }
    }
    else { // end server notification
        LOG.debug("Not connected to network.");
        setServerNotified(false);
    }

    LOG.debug("Exiting addressChangeHandler");
    return ret;
}




/**
 * Reads the 'push' registry value from CTPConfig, possible values:
 * 0  -> None
 * 1  -> STP
 * 2  -> CTP
 * 3  -> STP with fallback to CTP
 */
int getPushTypeFromRegistry() {

    CTPConfig config(APPLICATION_URI);
    config.readCTPConfig();
    return config.getPush();
}



/**
 * Starts the program "startsync.exe" with params:
 * "manual <sourcename1, sourcename2,...>" where sourcenames are
 * retrieved from the SyncNotification passed.
 * @note      the process is started and then the control is returned
 *            immediately to the caller (no wait on the process started)
 * @param sn  pointer to the SyncNotification object to read
 * @return    the PID of process started, a value < 0 in case of errors
 */
int startSyncFromSAN(SyncNotification* sn) {

    if (!sn) {
        LOG.error("CTP notification error: SyncNotification is NULL");
        return -1;
    }

    // Get number of sources to sync
    int n = sn->getNumSyncs();
    if (!n) {
        LOG.info("CTP notification error: no sources to sync from server");
        return -2;
    }

    //
    // Call "startsync.exe" with command-line parameters (sequence of source names)
    //
    wstring app     = L"startsync.exe";
    wstring cmdLine = L"manual";
    int count = 0;
    for (int i=0; i<n; i++) {
        SyncAlert* sync = sn->getSyncAlert(i);
        if(!sync) {
            LOG.error("CTP notification error: no SyncAlert in SyncNotification");
            return -3;
        }

        WCHAR* srcname = getSSourceName(APPLICATION_URI, sync->getServerURI());
        if (srcname) {
            if (count == 0) { cmdLine += L" "; }
            else            { cmdLine += L","; }
            cmdLine += srcname;
            delete [] srcname;
            count++;
        } 
        else {
            LOG.error("CTP notification error: no source found from server notification request: %s", sync->getServerURI());
            continue;
        }
    }

    if (count == 0) {
        // 0 sources to sync -> out
        LOG.info("No sources to sync");
        return NULL;
    }
    
    //
    // Start the sync process: "startsync.exe manual <sourcename1, sourcename2,...>"
    //
    int pid = 0;
    if (askConfirm()) {
        LOG.info("Starting sync: '%ls %ls'", app.c_str(), cmdLine.c_str());
        pid = startProgram(app.c_str(), cmdLine.c_str());
    }
    return pid;
}



/**
 * Utility to extract the nonce parameter from CTPMessage passed.
 * The nonce is expected to be the first parameter.
 * @param authStatusMsg  the CTPMessage to analyze
 * @return               the nonce extracted (as a new allocated char*), 
 *                       NULL il nonce param not found
 */
char* extractNonceParam(CTPMessage* authStatusMsg) {

    if (authStatusMsg->params.size() == 0) {
        //LOG.debug("Error receiving Status message: NONCE param is missing");
        return NULL;
    }

    list<CTPParam>::iterator param = authStatusMsg->params.begin();
    if ((*param).getParamCode() == P_NONCE) {
        int nonceLen = (*param).getValueLength();
        void* nonce  = (*param).getValue();

        // Return buffer is new allocated!
        return stringdup((char*)nonce, nonceLen);
    }
    else {
        return NULL;
    }
}


/**
 * Utility to extract the message length from a CTP package.
 * The message length is stored in the first 2 bytes of the CTP package.
 * @param  package    the input package
 * @param  packageLen the length of the passed package
 * @return            the message length
 */
int extractMsgLength(const char* package, int packageLen) {

    if (packageLen < 2) {
        LOG.error("Unable to read the package length: not enough bytes received (%d)", packageLen);
        return 0;
    }

    int messageLen  = (int)((unsigned char)package[0]);
    int messageLen1 = (int)((unsigned char)package[1]);

    messageLen <<= 8;
    messageLen = messageLen | messageLen1;
    //LOG.debug("messageLen = %d", messageLen);
    return messageLen;
}


/**
 * Print the message written in exadecimal code.
 */
void hexDump(char *buf, int len){

    if (LOG.getLevel() < LOG_LEVEL_DEBUG) {
        return;
    }

    //int i=0, byte;
    //for(i=0;i<len;i++){
    //    if(i%8==0){
    //        LOG.debug("%08x: ", i);
    //    }
    //    byte = buf[i];
    //    LOG.debug("%02x%s", byte  & 0xFF, (i%2)? " ":"");
    //}

    char* tmp = new char[len*3 + 3];
    tmp[0] = '[';
    int pos = 1;
    for (int i=0; i<len; i++) {
        sprintf(&tmp[pos], "%02x ", buf[i]);
        pos += 3;
    }
    tmp[pos-1] = ']';
    tmp[pos] = 0;
    LOG.debug("%s", tmp);
    delete [] tmp;
}

