/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */


// startsync.cpp - main module for the sync application
//

// Windows Header Files:
#include <windows.h>
#include <initguid.h>
#include <Notify.h>
#include <Pmpolicy.h>

#include "base/startcmd.h"
#include "base/util/utils.h"

#include "pim/SettingFunctions.h"
#include "pim/maincpp.h"
#include "notify/addresschange.h"
#include "pim/difftime.h"
#include "usernotify.h"
#include "base/Log.h"
#include "pushmessage.h"
#include "notify/util.h"
#include "notify/timed_msgbox.h"
#include "spds/constants.h"
#include "OTAConfigMessage.h"
#include "OTAConfigMessageHandler.h"
#include <cemapi.h>
#include "pim/utils.h"
#include <string>
#include <list>
#include "customization.h"
using namespace std;

//
// USE the pim/ClientSettings.h. THis must be shared with the same ClientSettings of the
// FunambolClient
//
#include "pim/ClientSettings.h"
#include "OutOfMemoryException.h"

#include "HwndFunctions.h"
#include "localizationUtils.h"

#define DIMCONF 128
#define DIM_PATH 256
#define NOT_RETURN_TO_MAIN_FORM -3
#define NO_SOURCES_FROM_TRANSPORT -12

#define CTP_QUEUE_PUSH      "queuePush"
#define CTP_QUEUE_LIST      "queueList"


#ifdef WIN32_PLATFORM_WFSP
#include "funresourcesp.h"
#endif
#ifdef WIN32_PLATFORM_PSPC
#include "funresourceppc.h"
#endif


#define PARAM_AVAILABLE         TEXT("available")
#define PARAM_REMOVENOTIF       TEXT("removeNotif")
#define PARAM_SCHEDULE          TEXT("schedule")
#define PARAM_CRADLE            TEXT("cradle")
#define PARAM_ADDRESSCHANGE     TEXT("addresschange")
#define PARAM_REGISTER          TEXT("register")
#define PARAM_DEREGISTER        TEXT("deregister")
#define PARAM_NOTIFY            TEXT("/notify")
#define PARAM_TRANSPORT         TEXT("transport")
#define PARAM_INCLUSIVE         TEXT("inclusive")
#define PARAM_MANUAL            TEXT("manual")
#define PARAM_TCPIP             TEXT("tcpip")
#define PARAM_REFRESHSERVER     TEXT("refresh-from-server")
#define PARAM_REFRESHCLIENT     TEXT("refresh-from-client")
#define PARAM_T_REFRESHSERVER   TEXT("transport-refresh-from-server")
#define PARAM_T_REFRESHCLIENT   TEXT("transport-refresh-from-client")

#define QUEUED_SYNC_DELAY       1       // Dalay time (seconds) before starting a queued sync.


//--------------------------------------------------------------------- Prototypes

static inline void Alert(const wchar_t *msg, const wchar_t *title=TEXT("Warning"));

static int getPollingTime();
static bool isServerNotified();
static void setServerNotified(bool val);

//static int polling(const wchar_t **sources);
static int schedule(bool notStartNow);
static int sync(const wchar_t **sources, const wchar_t **ids);
static int doSync(const char* mode, const wchar_t **sources, const wchar_t **ids);
int checkRefreshFromServer(const wchar_t** sources);

static bool g_debug = false;
static int retValue = 0;

void setValuesInRegistry(const wchar_t* name, const wchar_t* value);
wchar_t** getEnabledSourcesFromConfig();
void setSourceOnReg(wchar_t** sources);

void updateQueueList (const WCHAR** sources);
wchar_t** readQueueList();

//--------------------------------------------------------------- Static Functions

static inline void Alert(const wchar_t *msg, const wchar_t *title)
{
#ifdef DEBUG
  	MessageBox (NULL, msg, title, MB_SETFOREGROUND |MB_OK);
#endif
}

/*
* This function save the process id passed by setProcessId or resetProcessId on the registry
* NOT use the function directly but use (re)setProcessID instead
*/
static void writeProcessIdOnRegistry(unsigned long processId) {
    wchar_t pid[128];
    wsprintf(pid, TEXT("%lu"), processId);
    setClientConfiguration(NULL, PROPERTY_SPDM_PID, pid, NULL);
}

static void resetProcessID() {
    writeProcessIdOnRegistry(0);
}

static void setProcessID() {
    DWORD processId = GetCurrentProcessId();
    writeProcessIdOnRegistry(processId);
}


static int getPollingTime()
{

    wchar_t buf[32];

    memset(buf, 0, 32*sizeof(wchar_t));
    getClientConfigurationInternal (TEXT(""), PROPERTY_SPDM_POLLING_NOTIFICATION,
                                 buf, FALSE);
    return wcstol(buf, NULL, 10);

}

static bool isServerNotified()
{
    wchar_t buf[10];

    memset(buf, 0, 10*sizeof(wchar_t));
    getClientConfigurationInternal (TEXT(""), PROPERTY_SPDM_SERVER_NOTIFIED,
                                     buf, FALSE);
    return ( wcscmp(buf, TEXT("1")) == 0);

}

static void setServerNotified(bool val)
{

    setClientConfiguration(
            TEXT(""),
            PROPERTY_SPDM_SERVER_NOTIFIED,
            ( (val) ? TEXT("1") : TEXT("0") ),
            NULL);

}

static void getAppName(wchar_t* appName) {

    wchar_t path[256];
    getClientConfigurationInternal (NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);
    wcscpy(appName, path);
    wcscat(appName, TEXT("\\"));
    wcscat(appName, SYNCAPP);

}
/*
* NOT_RETURN_TO_MAIN_FORM: is the value to avoid to send a message for the result page.
*/


static int schedule(bool notStartNow)
{
    long t = getPollingTime();

    WCHAR appName[MAX_PATH];
    getAppName(appName);
    SYSTEMTIME lpTime;
    SYSTEMTIME pollTime;

    LOG.debug("Schedule start");

    //if (t > 0 && !isServerNotified()) {
    if (t > 0) {

        if (notStartNow == false) {
            LOG.debug("Re-scheduling");
            //
            // Schedule the polling before running the sync.
            // it needs to avoid disable of polling if the user stops the sync.
            //
            GetLocalTime(&lpTime);
            pollTime = DT_AddDiff(nano100SecInMin, t, &lpTime);
            CeRunAppAtTime(appName,&pollTime);

            LOG.debug("sync");
            //retValue = sync(NULL, NULL);
            retValue = NOT_RETURN_TO_MAIN_FORM;
            LOG.debug("Re-scheduling");
            LOG.debug(_wcc(appName));
        }
        // The sync start
        else {
            GetLocalTime(&lpTime);
            pollTime = DT_AddDiff(nano100SecInMin, t, &lpTime);
            if (CeRunAppAtTime(appName,&pollTime)) {
                retValue = NOT_RETURN_TO_MAIN_FORM;
            }
        }

    }
    if (t == 0) {
        CeRunAppAtTime(appName,0);
        retValue = NOT_RETURN_TO_MAIN_FORM;
    }
    LOG.debug("Schedule end");
    return retValue;
}


static int sync(const wchar_t **sources = NULL, const wchar_t **ids = NULL)
{
    return doSync(NULL, sources, ids);
}

static int recover(const char* mode, const wchar_t **sources)
{
    return doSync(mode, sources, NULL);
}

//static int inclusive(const wchar_t **sources)
//{
//    doSync(mode, sources, NULL);
//}

/**
 * @param sources - syncronize the specified sources. Default is NULL: it means the sources
 *                to sync are the one set in the registry
 * @param ids - the ids of the mail to be downloaded with a inclusive filter
 *              usually it is NULL. Not null only with the "inclusive" command line
 */
static int doSync(const char* mode, const wchar_t **sources, const wchar_t **ids)
{
    HANDLE hMutex;  //Handle to mutex...


    // Create mutex here
    hMutex = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress") );

    switch(GetLastError()) {
        case ERROR_SUCCESS:
            LOG.debug("Mutex created.");
            break;
        case ERROR_ALREADY_EXISTS:
            // Save the sources to sync into a queue: sync will start at the end
            // of current sync.
            LOG.info("Another sync in progress, current syncis queued");
            updateQueueList(sources);
            retValue = ERR_ANOTHER_SYNC;
            return retValue;
        default:
            LOG.error("Failed to create mutex");
            return -1;
    }
    /*
    wnd = HwndFunctions::getWindowHandle();

    // Moved in maincpp
    if (wnd) {
        HwndFunctions::closePreviousMsgBox();
        SendMessage(wnd, (WM_APP+4), NULL, NULL ); //-- send starting sync message to UI
    } else {
        //MessageBox (NULL, TEXT("wnd empty"), TEXT ("test"), MB_SETFOREGROUND |MB_OK);
    }
    */
    wchar_t path[DIM_PATH];
    getClientConfigurationInternal(NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);

    HANDLE handleBubble = CreateThread(NULL, 0, createBubble, NULL, 0, NULL);
    if (handleBubble == 0) {
        LOG.error("CreateThread failed: <0x%08x>\n", GetLastError());
    }
    WaitForSingleObject(handleBubble, 0);

    setProcessID();
    retValue = synchronize(path, sources, ids, mode);

    // Start automatically the queued sync.
    // The 'queueList' value is cleared in registry, but a new sync notification
    // could be received during the sync.
    wchar_t** sList = NULL;
    while (sList = readQueueList()) {
        LOG.info("Starting a QUEUED sync in %d seconds...", QUEUED_SYNC_DELAY);
        SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_STARTSYNC_ENDED, 0, 0);
        Sleep(QUEUED_SYNC_DELAY * 1000);
        retValue = synchronize(path, (const wchar_t**)sList, ids, mode);
        for (int i=0; sList[i]; i++) {
            delete sList[i];
        }
        sList = NULL;
    }

    resetProcessID();

    CloseHandle( hMutex );
    LOG.debug("Mutex released.");

    return retValue;
}

static int notify(const wchar_t **sources=NULL)
{
    AN_ResponseCode responseCode = notifyAddressChange(TEXT(APPLICATION_URI));

    if(responseCode == AN_AddressAccepted ){
        setServerNotified(true);
    }
    else {
        setServerNotified(false);
    }
    return NOT_RETURN_TO_MAIN_FORM;
}

/**
 * Parse the command line
 *
 * @param cmdline - the command line (syntax: command [source,source,...])
 *
 * @param command (out) - the command name
 * @param sources (out) - a NULL terminated array of source names
 */
static int parseCmdLine(const LPTSTR cmdline,
                        wchar_t **command,
                        wchar_t ***sources)
{
    if( wcsstr(cmdline, TEXT("debug ")) ) {
        *command = wstrdup(cmdline+wcslen(TEXT("debug ")));
        g_debug = true;
    }
    else {
        *command = wstrdup(cmdline);
        g_debug = false;
    }

    *sources = NULL;

    wchar_t *args = wcschr(*command, ' ');
    if(!args) {
        // in this case comand == cmdline and sources is null
        return 0;
    }
    // terminate command and point to the start of arguments
    args[0] = 0; args++;
    if (!args[0]) {
        // empty args after a space, like before
        return 0;
    }
    int argc;
    wchar_t *arg = args;
    // Count arguments
    for(argc = 1; (arg=wcschr(arg+1, ',')) != 0; argc++);

    // Get arg list
    wchar_t **srclist = new wchar_t *[argc+1];
    arg=args;
    int i;

    for(i=0; i<argc; i++) {
        if(!arg || !arg[0])
            return -1;
        srclist[i]=arg;

        arg=wcschr(arg, ',');
        if(arg) {
            arg[0]=0; arg++;
        }
    }
    srclist[i]=0;
    *sources=srclist;

    return 0;
}

static void exitHandler() {
    if (retValue != ERR_ANOTHER_SYNC) {
        removeNotif();
        HwndFunctions::closePreviousMsgBox();
        HWND hwnd = HwndFunctions::getWindowHandle();
        //HWND hwnd2 = HwndFunctions::getAccountWindowHandle();
        //::SendMessage(hwnd2, WM_COMMAND,   (WPARAM) 6, NULL);

        if (hwnd && retValue != NOT_RETURN_TO_MAIN_FORM) {
            SendMessage(hwnd, (WM_APP+3), NULL, (LPARAM)retValue ); // send msg startsync ended
        } else if (retValue == -40) {
            MessageBox (NULL, getLocalizationUtils()->getLocalizationString(IDS_ERR_OUT_OF_MEMORY), getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT), MB_SETFOREGROUND | MB_ICONWARNING |MB_OK);
            //showMessage(IDS_ERR_OUT_OF_MEMORY);
        } else if (retValue == -50) {
            MessageBox (NULL, getLocalizationUtils()->getLocalizationString(IDS_ERR_LOW_MEM), getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT), MB_SETFOREGROUND | MB_ICONWARNING |MB_OK);
            //showMessage(IDS_ERR_LOW_MEM);
        }
    }
    if (retValue == -6) { // sync stopped by the user
        // reset the pid property
        resetProcessID();
    }
}

//------------------------------------------------------------ Public functions
void quit(int value) {
    retValue = value;
    exit(retValue);
}




//------------------------------------------------------------------------ MAIN
int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPTSTR    lpCmdLine,
                   int       nCmdShow) {
         
    _set_new_mode(1);
    _set_new_handler(operatorNewHandler);
    // Set the exit handler to remove the icon
    atexit(exitHandler);
    try {
        //MessageBox (NULL, TEXT("stsync"), TEXT("title"), MB_SETFOREGROUND |MB_OK);
        wchar_t *command, **sources;

        HRESULT         hr              = NOERROR;
        HANDLE          hevtMsgAvailable= NULL;
        HPUSHROUTER     hPushRouter     = NULL;
        PUSHMSG         PushMsg         = {0};

        PowerPolicyNotify(PPN_UNATTENDEDMODE, TRUE);
        //SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_BELOW_NORMAL);
        
        if(g_debug){
            Alert(lpCmdLine);
        }

        if (parseCmdLine(lpCmdLine, &command, &sources)) {
            Alert(TEXT("Syntax error in command line"), TEXT("StartSync"));
            return 1;
        }
        
        if(!(wcscmp(command,PARAM_AVAILABLE) == 0 ||
            wcscmp(command, PARAM_REMOVENOTIF) == 0 ||
            wcscmp(command, PARAM_SCHEDULE) == 0 ||
            wcscmp(command, PARAM_CRADLE) == 0 ||
            wcscmp(command, PARAM_ADDRESSCHANGE) == 0 ||
            wcscmp(command, PARAM_REGISTER) == 0 ||
            wcscmp(command, PARAM_DEREGISTER) == 0 ||
            wcscmp(command, PARAM_NOTIFY) == 0 ||
            wcscmp(command, PARAM_TRANSPORT) == 0 ||
            wcscmp(command, PARAM_INCLUSIVE) == 0 ||
            wcscmp(command, PARAM_MANUAL) == 0 ||
            wcscmp(command, PARAM_REFRESHSERVER) == 0 ||
            wcscmp(command, PARAM_REFRESHCLIENT ) == 0 ||
            wcscmp(command, PARAM_T_REFRESHSERVER) == 0 ||
            wcscmp(command, PARAM_T_REFRESHCLIENT ) == 0 ||
            wcscmp(command, c_szParams) == 0 ||
            wcscmp(command, PARAM_TCPIP) == 0 ||
            wcscmp(command, APP_RUN_AT_TIME) == 0 
            ) ) {

                quit(0);
        }
        /*
        * These two following test are only to understand and make right 
        * modification. So the Cradle action is normalized and the RunAppAtTime
        * currently only schedules the next time sync and leaves to the flow
        * the right choice.
        */
        if (wcsicmp(command, PARAM_NOTIFY) == 0 ){
            //This if for the on cradle notification
            delete [] command;
            command = wstrdup(TEXT("cradle"));
            //command = TEXT("cradle");
            if (sources){
                delete [] sources;
            }
            sources = NULL;
        } else if(wcscmp(command, APP_RUN_AT_TIME) == 0 ){
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            schedule(false);
        } 
        
        if(wcscmp(command, PARAM_AVAILABLE) == 0) {
            getAvailability();
        } else if(wcscmp(command, PARAM_REMOVENOTIF) == 0) {
            removeNotif();
            retValue = NOT_RETURN_TO_MAIN_FORM;            
        } else if(wcscmp(command, PARAM_SCHEDULE) == 0) {
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            schedule(true);            
        } else if (wcscmp(command, PARAM_ADDRESSCHANGE) == 0) {
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            notify((const wchar_t **)sources);            
        } else if(wcscmp(command, PARAM_REGISTER) == 0) {            
            WCHAR c_szPath[MAX_PATH];
            getAppName(c_szPath); // /Program files/Funambol/startsync.exe
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            hr = PushRouter_RegisterClient(c_szContentType, c_szAppId, c_szPath, c_szParams);
            if (hr == S_OK) {
                LOG.debug("Wap push client registered successfully");
            } else if (hr == PUSHRTR_E_ALREADYREGISTERED) {
                LOG.debug("Wap push client already registered");
            } else {
                LOG.error("Error to register wap push client");
            }
            retValue = NOT_RETURN_TO_MAIN_FORM;
            
        } else if(wcscmp(command, PARAM_DEREGISTER) == 0) {
            //
            // unregister the startsync from the pushrouter
            //
            hr = PushRouter_UnRegisterClient(c_szContentType, c_szAppId);
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            if (hr == S_OK) {
                LOG.debug("Wap push client unregistered successfully");
            } else if (hr == PUSHRTR_E_NOTFOUND) {
                LOG.debug("Wap push client not found. Arleady unregistered");
            } else {
                LOG.error("Error to unregister wap push client");
            }
            retValue = NOT_RETURN_TO_MAIN_FORM;
        } else if(wcscmp(command, c_szParams) == 0) {
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            memset(&PushMsg, 0, sizeof(PUSHMSG));
            PushMsg.cbSize = sizeof(PUSHMSG);

            hr = PushRouter_Open(c_szContentType, c_szAppId, &hevtMsgAvailable, &hPushRouter);
            if (hr != S_OK) {
                LOG.error("Wap push: PushRouter_Open - it is not possible to register the client.");
            }
            while(WaitForSingleObject(hevtMsgAvailable, 10000) != WAIT_TIMEOUT) {
                hr = PushRouter_GetMessage(hPushRouter, &PushMsg);
                if (hr != S_OK) {
                    LOG.error("Error in PushRouter_GetMessage wap push client");
                    break;
                }
                wchar_t** ss;
                wchar_t** ws = NULL;
                int ret = handleWapPushMessage(&PushMsg, &ws);

                int i = 0;
                int a = 0;
                while(ws[i]){
                    char* srctocmp = toMultibyte(ws[i]);
                    if (!(strcmp(getRegConfig()->getConfigSourcesParameter(srctocmp, "sync"), "none") == 0)){
                        a++;
                    }
                    i++;
                    if (srctocmp){
                        delete [] srctocmp;
                    }
                }

                if(a != 0){
                    ss = new wchar_t*[a+1];

                    i = 0;
                    a = 0;
                    int sizews = 0;
                    while(ws[i]){

                        char* srctocmp = toMultibyte(ws[i]);
                        if (!(strcmp(getRegConfig()->getConfigSourcesParameter(srctocmp, "sync"), "none") == 0)){
                            sizews = wcslen(ws[i])+ 1;
                            ss[a] = new wchar_t[sizews];
                            wcscpy(ss[a], ws[i]);
                            a++;
                        }
                        if ( srctocmp ){
                            delete [] srctocmp;
                        }
                        i++;
                    }
                    ss[a] = 0;
                }
                else{
                    ss = NULL;
                }
                if (ret != 0) {
                    LOG.error("Wap push: error in handleWapPushMessage message");
                    break;
                }
                if (ss) {
                    if (askConfirm()) {
                                                
                        bool found = false;
                        int i = 0;
                        for (i = 0; ss[i]; i++) {
                            if (wcsstr(ss[i], L"mail") != 0) {
                                found = true;
                            }
                        }
                        if (found) {
                             setSourceOnReg(ss);
                             MailSyncMessages(_T(PROVIDER), MCF_ACCOUNT_IS_NAME | MCF_RUN_IN_BACKGROUND); 
                             return retValue = NOT_RETURN_TO_MAIN_FORM;                             
                        }

                        sync((const wchar_t **)ss);
                    }
                }
                PushRouter_FreeMessage(&PushMsg);
                memset(&PushMsg, 0, sizeof(PUSHMSG));
                PushMsg.cbSize = sizeof(PUSHMSG);                
            }
        } else if (wcscmp(command, PARAM_INCLUSIVE) == 0) {
            
            // just to know the number of ids
            unsigned int i = 0;
            for (i = 0; sources[i]; i++) {
                // the position start from 0 [that is mail, the name of the source]
                // [1], [2].. contains the ids
                //
            }
            // the name of the source: it should be "mail"
            wchar_t* mailSourceName = wstrdup(sources[0]);

            // the buffer for the ids
            const wchar_t ** ids = new const wchar_t*[i];

            for (unsigned j = 0; j < i; j++) {
                ids[j] =  wstrdup(sources[j + 1]);
            }
            delete [] sources;

            sources = new wchar_t* [2];
            sources[0] = mailSourceName;
            sources[1] = 0;
            sync((const wchar_t **)sources, (const wchar_t **)ids);

        } else if (wcscmp(command, PARAM_TRANSPORT) == 0 ||
                   wcscmp(command, PARAM_T_REFRESHSERVER) == 0 ||
                   wcscmp(command, PARAM_T_REFRESHCLIENT ) == 0) {

            if (strcmp(getRegConfig()->getConfigSourcesParameter("mail", "sync"), "none") == 0) {
                int ret = TimedMessageBox(NULL, 
                    getLocalizationUtils()->getLocalizationString(IDS_MAIL_DISABLED),
                    getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT), 
                    MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);
                if(ret == IDYES){
                    getRegConfig()->setConfigSourcesParameter( "mail" , "sync" , "two-way" );
                    getRegConfig()->setDirty(CS_DIRTY_SOURCE_ENABLED);
                    getRegConfig()->saveDirty();
                } else {
                    return 1;
                }
            }
            if( wcscmp(command, PARAM_T_REFRESHSERVER) == 0) {
                if (sources == 0) { //////****************
                    LOG.error("%S call without sources", PARAM_T_REFRESHSERVER);
                    retValue = NOT_RETURN_TO_MAIN_FORM;
                } else {
                    recover("refresh-from-server", (const wchar_t **)sources);
                }

            } else if (wcscmp(command, PARAM_T_REFRESHCLIENT ) == 0) {
                if (sources == 0 ){ //////****************
                    LOG.error("%S call without sources", PARAM_T_REFRESHCLIENT);
                    retValue = NOT_RETURN_TO_MAIN_FORM;
                } else {
                    recover("refresh-from-client", (const wchar_t **)sources);
                }

            } else if (sources) {
                sync((const wchar_t **)sources);

            } else if (!sources) {
                LOG.error("Transport call without sources");
                retValue = 0;
            }
        } else {
            if (!sources) {
                sources = getEnabledSourcesFromConfig();
                if (!sources) {
                    LOG.error("No sources in the config are selected to be synced");
                    quit(1);
                }                
            }
            bool flagmail = false;
            for(int i = 0; sources[i]; i++){
                if (wcsicmp(sources[i], TEXT("mail")) == 0){
                    flagmail = true;
                }
            }
            if(!flagmail) {
                if (wcsicmp(command, TEXT("refresh-from-server")) == 0) {
                    if (sources == 0) {
		                quit(1);
	                }
                    recover("refresh-from-server", (const wchar_t **)sources);

                } else if (wcscmp(command, TEXT("refresh-from-client")) == 0) {
                    if (sources == 0) {
		                quit(1);
                    }
                    recover("refresh-from-client", (const wchar_t **)sources);
                } else { // manual parameter
                    sync((const wchar_t **)sources);
                }
            } else {
                if (wcscmp(command, TEXT("refresh-from-server")) == 0 || 
                    wcscmp(command, TEXT("refresh-from-client")) == 0){
                    
                    wstring value = TEXT("transport-");
                    value += command;
                    setValuesInRegistry(TEXT("transportCommand"), value.c_str());
                }
                setSourceOnReg(sources);
                MailSyncMessages(ACCOUNT_NAME, MCF_ACCOUNT_IS_NAME | MCF_RUN_IN_BACKGROUND);
                return retValue = NOT_RETURN_TO_MAIN_FORM;
            }
        }
        if (hPushRouter) {
            PushRouter_Close(hPushRouter);
            PushRouter_FreeMessage(&PushMsg);
        }

        PowerPolicyNotify(PPN_UNATTENDEDMODE, FALSE);

        if (command){
            delete [] command;
        }
        if (sources){
            delete [] sources;
        }

    }//end try
    catch (OutOfMemoryException& e) {

        int k = e.getMemExcErrorCode();
        if (k == -50) {
            LOG.error("Low Storage Memory Exception.");
            quit(k);
        }
        else if (k == -1) { // error due to Out of memory
            LOG.error("Out of memory. Required %lu ", e.getMemExcMemoryRequired());
            quit(-40);
        } else { // error due to Out of memory. Not -1 set but supposed
            LOG.error("Out of memory");
            quit(-40);
        }
    }
    catch (...) {
        LOG.error("Generic failure");
        quit(-2);
    }

    return retValue;
}

void setValuesInRegistry(const wchar_t* name, const wchar_t* value) {
    
    HKEY key;
    DWORD res;
    int sizew = 0;

    RegCreateKeyEx(
        HKEY_LOCAL_MACHINE,
        TEXT("Software\\") ROOT_CONTEXT_W,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS,
        NULL,
        &key,
        &res
        );
    
    if (key == 0) {
        goto finally;
    }
        
    RegSetValueEx(
        key,
        name,
        NULL,
        REG_SZ,  // we currently support only strings
        (UCHAR*)value,
        (wcslen(value)+1)*sizeof(WCHAR)
        );

    RegFlushKey(key);

finally:

    if (key != 0) {
        RegCloseKey(key);
    }    
}


/**
* Set a list of sources that has to be synced passing through 
* the SyncMLTransport to take advantage of the mail notification.
* This is done only if the mail source is in the list of sources 
* to be synced
*/
void setSourceOnReg(wchar_t** sources) {
    
    wchar_t* value = NULL;
    int sizew = 0;
   
    for (int i = 0; sources[i]; i++ ){
        sizew += wcslen (sources[i]) + 1;
    }

    value = new wchar_t[sizew];
    
    for (int i = 0; sources[i]; i++ ){
        if (i == 0){
            wcscpy(value, sources[i]);
        } else {
            wcscat(value, TEXT(","));
            wcscat(value, sources[i]);
        }
    }
    
    setValuesInRegistry(TEXT("transportSources"), value);
   
    if (value) {
        delete [] value; value = NULL;
    }
}

/**
* Read the enabled sources from the configuration and return an array null terminated
* with the names of the sources to be synced
*/
wchar_t** getEnabledSourcesFromConfig() {
    
    int i = 0, count = 0;
    wchar_t** sourcesfr = NULL;

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("mail", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("contact", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("calendar", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("task", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("note", "sync"), "none") == 0)) {
        count++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("briefcase", "sync"), "none") == 0)) {
        count++;
    }
    if (count == 0) {
        return sourcesfr;
    }

    sourcesfr = new wchar_t*[count + 1];                

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("mail", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("mail"));
        i++;
    }

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("contact", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("contact"));        
        i++;
    }

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("calendar", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("calendar"));
        i++;
    }

    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("task", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("task"));                         
        i++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("note", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("note"));                          
        i++;
    }
    if(!(strcmp(getRegConfig()->getConfigSourcesParameter("briefcase", "sync"), "none") == 0)){
        sourcesfr[i] = wstrdup(TEXT("briefcase"));  
        i++;
    }
    sourcesfr[i] = 0;
    return sourcesfr;

}



/**
 * Reads and updates the property 'queueList' from Win registry with
 * the list of sources passed as parameter.
 * The 'queueList' value is a bitmask value used to identify which sources
 * have to be synced on the next (queued) sync. This value will be read
 * at the end of the sync, and a new sync will automatically start (if value != 0).
 * The 'queueList' bitmask is formatted as:
 *      bit 0  ->  mail
 *      bit 1  ->  contact
 *      bit 2  ->  calendar
 *      bit 3  ->  task
 *      bit 4  ->  note
 *      bit 5  ->  briefcase
 *
 * @param sources  the NULL terminated array of source names
 */
void updateQueueList(const wchar_t** sources) {

    if (!sources) {
        return;
    }
    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    char buf[4];

    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;

    // Check if we use the queue (queuePush = "1")
    value = node->readPropertyValue(CTP_QUEUE_PUSH);
    if ( !value || strcmp(value, "1") ) {
        // Not using queuePush
        goto finally;
    }
    
    // Read current value of 'queueList' from registry
    if (value) {
        delete [] value; value = NULL;
    }
    value = node->readPropertyValue(CTP_QUEUE_LIST);
    if (!value) value = stringdup("0");
    mask = atoi(value);

    //
    // Update the bit-mask with current sources
    //
    for (int i=0; sources[i]; i++) {
        if (!wcscmp(sources[i], MAILS_NAME)) {
            mask = mask | 1;
        }
        else if (!wcscmp(sources[i], CONTACTS_NAME)) {
            mask = mask | 2;
        }
        else if (!wcscmp(sources[i], APPOINTMENTS_NAME)) {
            mask = mask | 4;
        }
        else if (!wcscmp(sources[i], TASKS_NAME)) {
            mask = mask | 8;
        }
        else if (!wcscmp(sources[i], NOTES_NAME)) {
            mask = mask | 16;
        }
        else if (!wcscmp(sources[i], FILES_NAME)) {
            mask = mask | 32;
        }
    }

    // Write 'queueList' value to registry
    sprintf(buf, "%d", mask);
    node->setPropertyValue(CTP_QUEUE_LIST, buf);
    node->setPropertyValue(CTP_QUEUE_PUSH, "1");

finally:
    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;
}



/**
 * Reads the property 'queueList' from Win registry and returns a
 * NULL terminated array of source names to sync (new buffer).
 * The 'queueList' is cleared at the end.
 * The 'queueList' value is a bitmask value used to identify which sources
 * have to be synced on the next (queued) sync. 
 * The 'queueList' bitmask is formatted as:
 *      bit 0  ->  mail
 *      bit 1  ->  contact
 *      bit 2  ->  calendar
 *      bit 3  ->  task
 *      bit 4  ->  note
 *      bit 5  ->  briefcase
 *
 * @return  the NULL terminated array of source names to sync
 */
wchar_t** readQueueList() {

    wchar_t** sources    = NULL;
    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    list<wstring> sourceList;
    list<wstring>::iterator it;

    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;

    // Check if we use the queue (queuePush = "1")
    value = node->readPropertyValue(CTP_QUEUE_PUSH);
    if ( !value || strcmp(value, "1") ) {
        // Not using queuePush
        goto finally;
    }
    
    // Read current value of 'queueList' from registry
    if (value) {
        delete [] value; value = NULL;
    }
    value = node->readPropertyValue(CTP_QUEUE_LIST);
    if (!value) {
        goto finally;
    }
    mask = atoi(value);
    if (mask <= 0) {
        goto finally;
    }


    // Create the NULL terminated array of source names 
    // (first need the number of sources, to allocate the WCHAR**)
    sourceList.clear();
    if (mask &  1)  sourceList.push_back(MAILS_NAME       );
    if (mask &  2)  sourceList.push_back(CONTACTS_NAME    );
    if (mask &  4)  sourceList.push_back(APPOINTMENTS_NAME);
    if (mask &  8)  sourceList.push_back(TASKS_NAME       );
    if (mask & 16)  sourceList.push_back(NOTES_NAME       );
    if (mask & 32)  sourceList.push_back(FILES_NAME       );

    int i=0;
    sources = new wchar_t*[sourceList.size()+1];
    it = sourceList.begin();
    while (it != sourceList.end()) {
        sources[i++] = wstrdup((*it).c_str());
        it++;
    }
    sources[i] = NULL;


    // Clear the 'queueList' value on registry.
    node->setPropertyValue("queueList", "0");
    node->setPropertyValue("queuePush", "1");

finally:

    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;

    return sources;
}
